# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['oka']

package_data = \
{'': ['*']}

install_requires = \
['PyJWT>=2.3.0,<3.0.0',
 'idict>=1.211203.1,<2.0.0',
 'python-dotenv>=0.19.2,<0.20.0',
 'requests>=2.26.0,<3.0.0']

setup_kwargs = {
    'name': 'oka',
    'version': '0.211202.4',
    'description': 'Python client for oka repository',
    'long_description': '![test](https://github.com/rabizao/oka/workflows/test/badge.svg)\n[![codecov](https://codecov.io/gh/rabizao/oka/branch/main/graph/badge.svg)](https://codecov.io/gh/rabizao/oka)\n<a href="https://pypi.org/project/oka">\n<img src="https://img.shields.io/pypi/v/oka.svg?label=release&color=blue&style=flat-square" alt="pypi">\n</a>\n![Python version](https://img.shields.io/badge/python-3.8%20%7C%203.9-blue.svg)\n[![license: GPL v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)\n\n[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.5501845.svg)](https://doi.org/10.5281/zenodo.5501845)\n[![arXiv](https://img.shields.io/badge/arXiv-2109.06028-b31b1b.svg?style=flat-square)](https://arxiv.org/abs/2109.06028)\n[![User Manual](https://img.shields.io/badge/doc-user%20manual-a0a0a0.svg)](https://rabizao.github.io/oka)\n[![API Documentation](https://img.shields.io/badge/doc-API%20%28auto%29-a0a0a0.svg)](https://rabizao.github.io/oka/api)\n\n# oka - Client for OKA repository\n[Latest version as a package](https://pypi.org/project/oka)\n\n[Current code](https://github.com/rabizao/oka)\n\n[User manual](https://rabizao.github.io/oka)\n\n[API documentation](https://rabizao.github.io/oka/api)\n\n## Overview\n`oka` is a client for Oka repository.\nIt also provides utilities to process data.\n\n## Installation\n### ...as a standalone lib\n```bash\n# Set up a virtualenv. \npython3 -m venv venv\nsource venv/bin/activate\n\n# Install from PyPI...\npip install --upgrade pip\npip install -U oka\npip install -U oka[full]  # use the flag \'full\' for extra functionality (recommended)\n\n# ...or, install from updated source code.\npip install git+https://github.com/rabizao/oka\n```\n    \n### ...from source\n```bash\nsudo apt install python3.8-venv python3.8-dev python3.8-distutils # For Debian-like systems.\ngit clone https://github.com/rabizao/oka\ncd oka\npython3.8 -m venv venv\nsource venv/bin/activate\npip install -e .\n```\n\n## Usage\n\n\n\n\n\n\n**Hello world**\n<details>\n<p>\n\n```python3\nfrom oka import Oka, generate_token, toy_df\n\n# Create a pandas dataframe.\ndf = toy_df()\nprint(df.head())\n"""\n   attr1  attr2  class\n0    5.1    6.4      0\n1    1.1    2.5      1\n2    6.1    3.6      0\n3    1.1    3.5      1\n4    3.1    2.5      0\n"""\n```\n\n```python3\n\n# Login.\ntoken = generate_token("http://localhost:5000")\nclient = Oka(token, "http://localhost:5000")\n\n# Store.\nid = client.send(df)\n\n# Store again.\nid = client.send(df)\n"""\nContent already stored for id iJ_e4463c51904e9efb800533d25082af2a7bf77\n"""\n\n# Fetch.\ndf = client.get(id)\n\nprint(df.head())\n"""\n   attr1  attr2  class\n0    5.1    6.4      0\n1    1.1    2.5      1\n2    6.1    3.6      0\n3    1.1    3.5      1\n4    3.1    2.5      0\n"""\n```\n\n</p>\n</details>\n\n\n\n\n\n\n\n\n\n**DataFrame by hand**\n<details>\n<p>\n\n```python3\nimport pandas as pd\nfrom oka import Oka, generate_token\n\n# Create a pandas dataframe.\ndf = pd.DataFrame(\n    [[1, 2, "+"],\n     [3, 4, "-"]],\n    index=["row 1", "row 2"],\n    columns=["col 1", "col 2", "class"],\n)\nprint(df.head())\n"""\n       col 1  col 2 class\nrow 1      1      2     +\nrow 2      3      4     -\n"""\n```\n\n```python3\n\n# Login.\ntoken = generate_token("http://localhost:5000")\nclient = Oka(token, "http://localhost:5000")\n\n# Store.\nid = client.send(df)\n\n# Store again.\nid = client.send(df)\n"""\nContent already stored for id f7_6b9deafec2562edde56bfdc573b336b55cb16\n"""\n\n# Fetch.\ndf = client.get(id)\n\nprint(df.head())\n"""\n       col 1  col 2 class\nrow 1      1      2     +\nrow 2      3      4     -\n"""\n```\n\n\n\n\n\n\n**Machine Learning workflow**\n<details>\n<p>\n\n```python3\nimport json\n\nfrom sklearn.ensemble import RandomForestClassifier as RF\n\nfrom idict import let, idict\nfrom idict.function.classification import fit, predict\nfrom idict.function.evaluation import split\nfrom oka import Oka, generate_token\n\n# Login.\ntoken = generate_token("http://localhost:5000")\ncache = Oka(token, "http://localhost:5000")\nd = (\n        idict.fromtoy()\n        >> split\n        >> let(fit, algorithm=RF, config={"n_estimators": 55}, Xin="Xtr", yin="ytr")\n        >> let(predict, Xin="Xts")\n        >> (lambda X: {"X2": X * X, "_history": ...})\n        >> [cache]\n)\ncache.send(d)\nprint(json.dumps(list(d.history.keys()), indent=2))\n"""\nssssssssssssssssssssssssssssssssssssssss\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n----setitem-------------------\n444444444444444444444444\n3333333333333333333333333\n[\n  "split----------------------sklearn-1.0.1",\n  "fit--------------------------------idict",\n  "predict----------------------------idict",\n  "RwMG040tZc3XNoJkwkBe6A1aIUGNQ4EAQVqi.uAl"\n]\n"""\n```\n\n```python3\n#\n# d.show()\n```\n\n```python3\n#\n# print(d.z)\n```\n\n```python3\n#\n# d.show()\n```\n\n```python3\n#\n# # A field \'_\' means this function is a noop process triggered only once by accessing one of the other provided fields."\n# d >>= (lambda _, X2, y: print("Some logging/printing that doesn\'t affect data...\\nX²=\\n", X2[:3]))\n# d.show()\n```\n\n```python3\n#\n# print("Triggering noop function by accessing \'y\'...")\n# print("y", d.y[:3])\n```\n\n```python3\n#\n# d.show()\n```\n\n```python3\n#\n# # The same workflow will not be processed again if the same cache is used.\n# d = (\n#         idict.fromtoy()\n#         >> split\n#         >> let(fit, algorithm=RF, config={"n_estimators": 55}, Xin="Xtr", yin="ytr")\n#         >> let(predict, Xin="Xts")\n#         >> (lambda X: {"X2": X * X})\n#         >> (lambda _, X2, y: print("Some logging/printing that doesn\'t affect data...", X2.head()))\n#         >> [cache]\n# )\n# d.show()\n```\n\n```python3\n#\n# cache.send(d)\n#\n# d = cache.get(d.id)\n# d.show()\n```\n\n\n</p>\n</details>\n\n\n\n\n\n\n\n## More info\nAside from the papers on [identification](https://arxiv.org/abs/2109.06028)\nand on [similarity (not ready yet)](https://), the [PyPI package](https://pypi.org/project/oka) \nand [GitHub repository](https://github.com/davips/rabizao/oka), \n<!-- one can find more information, at a higher level application perspective,  -->\nA lower level perspective is provided in the [API documentation](https://rabizao.github.io/oka).\n\n\n\n## Grants\nThis work was supported by Fapesp under supervision of\nProf. André C. P. L. F. de Carvalho at CEPID-CeMEAI (Grants 2013/07375-0 – 2019/01735-0).\n\n.>>>>>>>>> outros <<<<<<<<<<<.\n',
    'author': 'Rafael Bizao',
    'author_email': 'rabizao@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
